/*
 * Project: Examples for GJA course
 * Authors: Ing. Jan Kouřil
 *          Ing. Jaroslav Dytrych idytrych@fit.vutbr.cz
 * File: MacAddressValidatorHandler.java
 * Description: Example of Web Service handlers - MAC Address validator
 */

/**
 * @file MacAddressValidatorHandler.java
 *
 * @brief Example of Web Service handlers - MAC Address validator
 */

package cz.vutbr.fit.knot.gja.ws.server;

import java.io.IOException;
import java.util.Iterator;
import java.util.Set;
import javax.xml.namespace.QName;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConstants;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import javax.xml.ws.soap.SOAPFaultException;

/**
 * MAC Address validator
 */
public class MacAddressValidatorHandler implements SOAPHandler<SOAPMessageContext> {

  /**
   * Handles message 
   * 
   * @param context
   * @return Returns whether continue in handler chain
   */
  @Override
  public boolean handleMessage(SOAPMessageContext context) {

    System.out.println("Server : handleMessage()......");

    Boolean isRequest = (Boolean) context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

    // For response message only, true for outbound messages, false for inbound
    if (!isRequest) {

      try {
        SOAPMessage soapMsg = context.getMessage();
        SOAPEnvelope soapEnv = soapMsg.getSOAPPart().getEnvelope();
        SOAPHeader soapHeader = soapEnv.getHeader();

        // If no header, add one
        if (soapHeader == null) {
          soapHeader = soapEnv.addHeader();
          // Throw exception
          generateSOAPErrMessage(soapMsg, "No SOAP header.");
        }

        // Get client MAC address from SOAP header
        Iterator it = soapHeader.extractHeaderElements(SOAPConstants.URI_SOAP_ACTOR_NEXT);

        // If no header block for next actor found? throw exception
        if (it == null || !it.hasNext()) {
          generateSOAPErrMessage(soapMsg, "No header block for next actor.");
        }

        // If no MAC address found? throw exception
        Node macNode = (Node) it.next();
        String macValue = (macNode == null) ? null : macNode.getValue();

        if (macValue == null) {
          generateSOAPErrMessage(soapMsg, "No mac address in header block.");
        }

        // If MAC address is not match, throw exception
	/*
         * eth0 if(!macValue.equals("E4-11-5B-F2-54-AA")){
         *   generateSOAPErrMessage(soapMsg, "Invalid mac address, access is denied."); 
         * }
         */

        if (!macValue.equals("AC-7B-A1-D4-90-E0")) {
          generateSOAPErrMessage(soapMsg, "Invalid mac address, access is denied.");
        }

        // wlan0
	/*
         * If(!macValue.equals("08-11-96-2F-D3-DC")){
         *   generateSOAPErrMessage(soapMsg, "Invalid mac address, access is denied."); 
         * }
         */

        // tracking
        soapMsg.writeTo(System.out);

      } catch (SOAPException e) {
        System.err.println(e);
      } catch (IOException e) {
        System.err.println(e);
      }
    }

    // continue other handler chain
    return true;
  }

  /**
   * Fault handler
   * 
   * @param context Message context
   * @return Returns whether continue in processing
   */
  @Override
  public boolean handleFault(SOAPMessageContext context) {
    System.out.println("Server : handleFault()......");

    return true;
  }

  /**
   * Called at the conclusion of a message exchange pattern
   * 
   * @param context Message context
   */
  @Override
  public void close(MessageContext context) {
    System.out.println("Server : close()......");
  }

  /**
   * Gets the header blocks that can be processed by this Handler instance
   * 
   * @return Returns header block (not implemented - returns null)
   */
  @Override
  public Set<QName> getHeaders() {
    System.out.println("Server : getHeaders()......");
    return null;
  }

  /**
   * Generates SOAP error message
   * 
   * @param msg Error message
   * @param reason Error reason
   */
  private void generateSOAPErrMessage(SOAPMessage msg, String reason) {
    try {
      SOAPBody soapBody = msg.getSOAPPart().getEnvelope().getBody();
      SOAPFault soapFault = soapBody.addFault();
      soapFault.setFaultString(reason);
      throw new SOAPFaultException(soapFault);
    } catch (SOAPException e) {
    }
  }
}  // public class MacAddressValidatorHandler
